import { Component, OnInit, AfterViewInit, ViewChild } from '@angular/core';
import { AfterContentInit } from '@angular/core/src/metadata/lifecycle_hooks';
import { Router } from '@angular/router';
import { Store, select } from '@ngrx/store';
import { AuthSelector } from 'packages/member/store';
import { RPCClientError } from '@loafer/ng-rpc/protocol';
import { Domain } from 'packages/domain/model';
import * as ListStore from '../../store/noauth-probe';
import { NoAuthProbeSelector } from '../../store';
import { NoAuthProbe } from '../../model';
import { ConfirmationService, Message } from 'primeng/primeng';
import { MessageService } from 'primeng/components/common/messageservice';

@Component({
  selector: 'of-noauth-list',
  templateUrl: './list.component.html',
  providers: [ConfirmationService, MessageService]
})
export class ListComponent implements OnInit, AfterContentInit {
  noAuthProbes$ = this.store.pipe(select(NoAuthProbeSelector.select('noAuthProbes')));
  noauthProbes: NoAuthProbe[];
  selected: NoAuthProbe = null;
  msgs: Message[];

  constructor(
    private router: Router,
    private store: Store<ListStore.State>,
    private confirmationService: ConfirmationService,
    private messageService: MessageService
  ) {
  }

  ngAfterContentInit() {
    this.store.select(AuthSelector.select('domain')).subscribe(
      (domain: Domain) => {
        this.store.dispatch(new ListStore.ReadAllByDomain(domain));
      }
    );

    this.noAuthProbes$.subscribe(
      (result: NoAuthProbe[]) => {
        if (result) {
          this.noauthProbes = result;
        }
      },
      (error: RPCClientError) => {
        console.log(error.response.message);
      }
    );
  }

  ngOnInit() {
    // Temporary Data
    this.noauthProbes = new Array();
    for (let i = 0; i < 10; i++) {
      const p: NoAuthProbe = {
        id: i,
        tempProbeKey: 'TempKey' + i,
        createDate: new Date(),
        description: 'Description' + i,
      };
      this.noauthProbes.push(p);
    }
  }

  handleSelect(selected: NoAuthProbe) {
    this.selected = selected;
  }

  onAcceptOrDeny(isAccept: boolean) {
    this.msgs = [];
    const title = isAccept ?
      'Are you sure to accept this Probe?' : 'Are you sure to deny this Probe';
    const message = isAccept ?
      'Start collecting data as a Probe.' : 'It will be permanently deleted.';

    this.confirmationService.confirm({
      header: title,
      message: message,
      icon: isAccept ? 'fa-check' : 'fa fa-trash',
      accept: () => {
        isAccept ? this.handleAccept() : this.handleDeny();
        this.msgs.push({ severity: 'error', summary: 'Failed', detail: '연동 안했지롱' });
      },
      reject: () => {
      }
    });
  }

  handleAccept() {
    this.store.dispatch(new ListStore.Accept(this.selected));
    this.noAuthProbes$.subscribe(
      (noAuthProbes: NoAuthProbe[]) => {
        if (noAuthProbes) {
          this.msgs.push({ severity: 'success', summary: 'Success', detail: '개굿' });
        }
      },
      (error: RPCClientError) => {
        this.msgs.push({ severity: 'error', summary: 'Failed', detail: error.response.message });
      }
    );
  }

  handleDeny() {
    this.store.dispatch(new ListStore.Deny(this.selected));
    this.noAuthProbes$.subscribe(
      (noAuthProbes: NoAuthProbe[]) => {
        if (noAuthProbes) {
          this.msgs.push({ severity: 'success', summary: 'Success', detail: '개굿' });
        }
      },
      (error: RPCClientError) => {
        this.msgs.push({ severity: 'error', summary: 'Failed', detail: error.response.message });
      }
    );
  }
}
