import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { RPCClientError } from '@loafer/ng-rpc/protocol';

import { Probe } from '../../model';
import { ProbeService } from '../../service/probe.service';

import {
  Read,
  ReadFailure,
  ReadSuccess,
  ActionType
} from './detail.action';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private probeService: ProbeService,
    private router: Router
  ) { }

  @Effect()
  read$: Observable<Action> = this.actions$
    .ofType(ActionType.Read)
    .map((action: Read) => action.payload)
    .switchMap(payload => this.probeService.read(payload.id))
    .map(probe => {
      return new ReadSuccess(probe);
    })
    .catch((error: RPCClientError) => {
      return of(new ReadFailure(error));
    });
}
