import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { RPCClientError } from '@loafer/ng-rpc/protocol';

import { DomainMember } from 'packages/domain/model';


import {
  ReadAllByDisplayItem,
  ReadAllByDisplayItemSuccess,
  ReadAllByDisplayItemFailure,
  ActionType,
} from './list.action';
import { SensorItemDependencyService } from '../../service/sensor-item-dependency.service';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private service: SensorItemDependencyService,
    private router: Router
  ) { }

  @Effect()
  readAllByDisplayItem$: Observable<Action> = this.actions$
    .ofType(ActionType.ReadAllByDisplayItem)
    .map((action: ReadAllByDisplayItem) => action.payload)
    .switchMap(payload => this.service.readAllByDisplayItem(payload))
    .map(list => {
      return new ReadAllByDisplayItemSuccess(list);
    })
    .catch((error: RPCClientError) => {
      return of(new ReadAllByDisplayItemFailure(error));
    });

}
